@@ -773,6 +773,46 @@ impl Array {
         Array::Bool(this.or(&that))
     }
 
+    /// Find the maximum value of each `stride` of this `Array`
+    pub fn reduce_max(&self, stride: u64) -> Result<Array> {
+        if self.len() as u64 % stride != 0 {
+            return Err(error(format!(
+                "cannot reduce an Array of length {} with stride {}",
+                self.len(),
+                stride
+            )));
+        }
+
+        fn reduce_block_dispatch<T: af::HasAfEnum>(block: &ArrayExt<T>, stride: u64) -> Array
+        where
+            Array: From<ArrayExt<T::InType>>,
+        {
+            reduce_block(block, stride, &mut |block| af::max(&block, 0).into()).into()
+        }
+
+        Ok(reduce!(self, reduce_block_dispatch, stride))
+    }
+
+    /// Find the maximum value of each `stride` of this `Array`
+    pub fn reduce_min(&self, stride: u64) -> Result<Array> {
+        if self.len() as u64 % stride != 0 {
+            return Err(error(format!(
+                "cannot reduce an Array of length {} with stride {}",
+                self.len(),
+                stride
+            )));
+        }
+
+        fn reduce_block_dispatch<T: af::HasAfEnum>(block: &ArrayExt<T>, stride: u64) -> Array
+        where
+            Array: From<ArrayExt<T::InType>>,
+        {
+            reduce_block(block, stride, &mut |block| af::min(&block, 0).into()).into()
+        }
+
+        Ok(reduce!(self, reduce_block_dispatch, stride))
+    }
+
     /// Compute the product of each `stride` of this `Array`
     pub fn reduce_product(&self, stride: u64) -> Result<Array> {
         if self.len() as u64 % stride != 0 {
@@ -813,6 +853,38 @@ impl Array {
         Ok(reduce!(self, reduce_block_dispatch, stride))
     }
 
+    /// Find the maximum element in this `Array`.
+    pub fn max(&self) -> Number {
+        fn max<T>(this: &ArrayExt<T>) -> Number
+        where
+            T: af::HasAfEnum + Default,
+            T::AggregateOutType: number_general::DType,
+            T::ProductOutType: number_general::DType,
+            ArrayExt<T>: ArrayInstanceMinMax<T>,
+            Number: From<T>,
+        {
+            this.max().into()
+        }
+
+        dispatch!(self, max)
+    }
+
+    /// Find the maximum element in this `Array`.
+    pub fn min(&self) -> Number {
+        fn min<T>(this: &ArrayExt<T>) -> Number
+        where
+            T: af::HasAfEnum + Default,
+            T::AggregateOutType: number_general::DType,
+            T::ProductOutType: number_general::DType,
+            ArrayExt<T>: ArrayInstanceMinMax<T>,
+            Number: From<T>,
+        {
+            this.min().into()
+        }
+
+        dispatch!(self, min)
+    }
+
     /// Calculate the cumulative product of this `Array`.
     pub fn product(&self) -> Number {
         fn product<T>(this: &ArrayExt<T>) -> Number
@@ -820,7 +892,7 @@ impl Array {
             T: af::HasAfEnum + Default,
             T::AggregateOutType: number_general::DType,
             T::ProductOutType: number_general::DType,
-            ArrayExt<T>: ArrayInstanceReduce<T>,
+            ArrayExt<T>: ArrayInstanceProduct<T>,
             Number: From<T::ProductOutType>,
         {
             this.product().into()
@@ -836,7 +908,7 @@ impl Array {
             T: af::HasAfEnum + Default,
             T::AggregateOutType: number_general::DType,
             T::ProductOutType: number_general::DType,
-            ArrayExt<T>: ArrayInstanceReduce<T>,
+            ArrayExt<T>: ArrayInstanceSum<T>,
             Number: From<T::AggregateOutType>,
         {
             this.sum().into()
@@ -1870,7 +1942,7 @@ impl<'en> en::IntoStream<'en> for DType {
     }
 }
 
-pub(super) fn reduce_block<T, B, R>(block: &ArrayExt<T>, stride: u64, reduce: &mut R) -> ArrayExt<B>
+pub(crate) fn reduce_block<T, B, R>(block: &ArrayExt<T>, stride: u64, reduce: &mut R) -> ArrayExt<B>
 where
     T: af::HasAfEnum,
     B: af::HasAfEnum,
@@ -2016,6 +2088,13 @@ mod tests {
         assert_eq!(a.pow(&b), [1.0, 4.0, 9.0][..].into());
     }
 
+    #[test]
+    fn test_min_and_max() {
+        let a: Array = [3, 1, 4, 2][..].into();
+        assert_eq!(a.min(), 1.into());
+        assert_eq!(a.max(), 4.into());
+    }
+
     #[test]
     fn test_sum() {
         let a: Array = [1, 2, 3, 4][..].into();
